package kd.bos.newdevportal.table;

import kd.bos.dataentity.utils.StringUtils;
import kd.bos.db.DB;
import kd.bos.db.DBRoute;
import kd.bos.entity.EntityMetadataCache;
import kd.bos.entity.datamodel.ListSelectedRow;
import kd.bos.entity.datamodel.ListSelectedRowCollection;
import kd.bos.form.control.events.ItemClickEvent;
import kd.bos.list.BillList;
import kd.bos.metadata.dao.MetadataDao;
import java.util.*;
import java.util.stream.Collectors;

/**
 * TableListPlugin 添加表注释和表字段注释
 * <p>
 * 添加表注释和表字段注释
 * </p>
 *
 * @author ww
 * @version 1.0
 */
public class TableListPlugin extends TableManagerListPlugin {
    public void registerListener(EventObject e) {
        super.registerListener(e);
        this.addItemClickListeners(new String[]{"toolbarap"});
    }
    @Override
    public void itemClick(ItemClickEvent evt) {
        String itemKey = evt.getItemKey();
        if ("nckd_remark".equals(itemKey)) {
            BillList billList = (BillList)this.getControl("billlistap");
            ListSelectedRowCollection list =  billList.getSelectedRows();
            Map<String, List<String >> entityIdGroups = new HashMap<>();
            // 统计成功和失败的次数
            int successCount = 0;
            int failureCount = 0;
            for (ListSelectedRow detail : list) {
                Object pk = detail.getPrimaryKeyValue();
                String pkStr = String.valueOf(pk);
                String[] info = pkStr.split("@@");
                if (info.length < 2) {
                    failureCount++; // 如果数据不正确，增加失败计数
                    continue; // 跳过此条记录，继续处理下一条
                }
                //获取数据字典明细
                String tablename = info[0];//表名称
                String entityId = info[1];//id
                String dbroute = EntityMetadataCache.getDataEntityType(MetadataDao.getEntityNumberById(info[1])).getDBRouteKey();
                // 根据 dbroute 将数据分组
                entityIdGroups.computeIfAbsent(dbroute, k -> new ArrayList<>()).add(pkStr);
            }
            // 打印每个分组的 tablename
            for (Map.Entry<String, List<String >> entry : entityIdGroups.entrySet()) {
                String dbroute = entry.getKey();
                List<String> pks = entry.getValue();
                // 使用 Stream 提取 tableNames 和 entityIds
                List<String> tableNames = pks.stream()
                        .map(pkStr -> pkStr.split("@@")) // 分割字符串
                        .filter(info -> info.length >= 2) // 过滤出有效数据
                        .map(info -> info[0]) // 提取表名称
                        .collect(Collectors.toList()); // 收集为 List

                List<String> entityIds = pks.stream()
                        .map(pkStr -> pkStr.split("@@")) // 分割字符串
                        .filter(info -> info.length >= 2) // 过滤出有效数据
                        .map(info -> info[1]) // 提取表名称
                        .collect(Collectors.toList()); // 收集为 List

                // 根据 entityId 列表生成 refEntityNum 列表
                List<String> refEntityNums = entityIds.stream()
                        .map(entityId -> StringUtils.isNotBlank(entityId) ? MetadataDao.getNumberById(entityId) : null)
                        .collect(Collectors.toList());

                DBTableProviderNew provider =new DBTableProviderNew();
                provider.setRoute(DBRoute.of(dbroute == null ? "sys" : dbroute));
                Map<String,DesignerTable> tableMap =  provider.getTableInfo_youhua(tableNames,refEntityNums);
                for (Map.Entry<String, DesignerTable> table : tableMap.entrySet()) {
                    String tablename = table.getKey();
                    DesignerTable designerTable = table.getValue();
                    String comment = designerTable.getName();
                    StringBuilder sql = new StringBuilder();
                    if (table != null) {
                        sql.append("/*dialect*/ COMMENT ON TABLE " + tablename + " IS '" + comment + "';");
                        //获取表字段
                        Iterator fields = designerTable.getCols().iterator();
                        while(fields.hasNext()) {
                            DesignerColumn col = (DesignerColumn)fields.next();
                            String f_fieldkey = col.getCode();//字段标识
                            String f_fieldname = col.getName();//字段名称
                            sql.append("/*dialect*/ COMMENT ON COLUMN " + tablename + "." +f_fieldkey + " IS '" + f_fieldname + "';");
                        }
                        // 获取SQL字符串
                        String sqlContent = sql.toString();

                        try {
                            //执行
                            DBRoute route = DBRoute.of(dbroute);
                            DB.execute(route, sqlContent);
                            successCount++; // 增加成功计数
                        } catch (Exception e) {
                            // 处理执行过程中可能出现的异常
                            e.printStackTrace(); // 可以替换为日志记录
                            failureCount++; // 增加失败计数
                        }
                    }else {
                        failureCount++; // 增加失败计数
                    }
                }
            }
            // 显示最终的成功和失败消息
            String message = String.format("执行完成：成功 %d 条，失败 %d 条", successCount, failureCount);
            this.getView().showMessage(message);
        }
    }
}
